.block[data-v-320582f6]{padding:30px 0;text-align:center;border-right:solid 1px var(--el-border-color);flex:1}.block[data-v-320582f6]:last-child{border-right:none}.block .demonstration[data-v-320582f6]{display:block;color:var(--el-text-color-secondary);font-size:14px;margin-bottom:20px}
